import ListErrors from './ListErrors';
import React from 'react';
import agent from '../agent';
import { connect } from 'react-redux';
import {
  SETTINGS_SAVED,
  SETTINGS_PAGE_UNLOADED,
  LOGOUT,
  DELETE_USER
} from '../constants/actionTypes';
import swal from 'sweetalert';
import ShowIcon  from '@material-ui/icons/Visibility';
import ShowOffIcon  from '@material-ui/icons/VisibilityOff';

class SettingsForm extends React.Component {
  constructor() {
    super();

    this.state = {
      image: '',
      username: '',
      bio: '',
      email: '',
      password: '',
      confirmPassword: ""
    };

    this.updateState = field => ev => {
      const state = this.state;
      const newState = Object.assign({}, state, { [field]: ev.target.value });
      this.setState(newState);
    };

    this.submitForm = ev => {
      ev.preventDefault();
      const regexp = /^\S*$/;//ei välilyöntejä
      const user = Object.assign({}, this.state);
      if(!regexp.test(user.password) || !regexp.test(user.confirmPassword)){
        swal(" Whitespace not allowed")
        return null;
      }
          
      if(user.password !== user.confirmPassword ) {
            
        swal("Passwords does not match  ")
        return null;       
      }
      
      if (!user.password) {
        delete user.password;              
      }
      else if(user.password.length < 4){
        swal("Password too short min 4 digits ")   
           return
         }
      else{
        swal({title: 'Password changed' ,
          timer: 1500,
          icon: 'success',
          showConfirmButton: false,})                   
      }
      
      this.props.onSubmitForm(user);
      
    };
  }

  componentWillMount() {
    if (this.props.currentUser) {
      Object.assign(this.state, {
        image: this.props.currentUser.image || '',
        username: this.props.currentUser.username,
        bio: this.props.currentUser.bio,
        email: this.props.currentUser.email
      });
    }
  }

  componentWillReceiveProps(nextProps) {
    if (nextProps.currentUser) {
      this.setState(Object.assign({}, this.state, {
        image: nextProps.currentUser.image || '',
        username: nextProps.currentUser.username,
        bio: nextProps.currentUser.bio,
        email: nextProps.currentUser.email
      }));
    }
  }

  render() {
    return (
      <form onSubmit={this.submitForm}>
        <fieldset>

          <fieldset className="form-group">
            <input
              className="form-control"
              type="text"
              placeholder="URL of profile picture"
              value={this.state.image}
              onChange={this.updateState('image')} />
          </fieldset>

          <fieldset className="form-group">
            <input
              className="form-control form-control-lg"
              type="text"
              placeholder="Username"
              value={this.state.username}
              onChange={this.updateState('username')} />
          </fieldset>

          <fieldset className="form-group">
            <textarea
              className="form-control form-control-lg"
              rows="8"
              placeholder="Short bio about you"
              value={this.state.bio}
              onChange={this.updateState('bio')}>
            </textarea>
          </fieldset>

          <fieldset className="form-group">
            <input
              className="form-control form-control-lg"
              type="email"
              placeholder="Email"
              value={this.state.email}
              onChange={this.updateState('email')} />
          </fieldset>

          <fieldset className="form-group">
            <input
              className="form-control form-control-lg"
              type={this.state.password ? "password":"text" }
              placeholder="New Password"
              //value={this.state.password}
              onChange={this.updateState('password')} />
            <span
              className="hideshowbtn" style={{top: '67%'}}
              onMouseDown={()=> this.setState({password:!this.state.password})}>
              {this.state.password ? <ShowOffIcon/>:<ShowIcon/>}
            </span>
          </fieldset>   
          
          <fieldset className="form-group">
            <input
              className="form-control form-control-lg"
              type={this.state.confirmPassword ? "password":"text" }
              placeholder="Confirm Password"
              // value={this.state.confirmPassword }              
              onChange={this.updateState('confirmPassword')} />
            <span
              className="hideshowbtn" style={{top: '76%'}}
              onMouseDown={()=> this.setState({confirmPassword:!this.state.confirmPassword})}>
              {this.state.confirmPassword ? <ShowOffIcon/>:<ShowIcon/>}
            </span>            
          </fieldset>

          <button
            className="btn btn-lg btn-primary pull-xs-right"
            type="submit"
            disabled={this.state.inProgress}>
            Update Settings
          </button>

        </fieldset>
      </form>
    );
  }
}

const mapStateToProps = state => ({
  ...state.settings,
  currentUser: state.common.currentUser
});

const mapDispatchToProps = dispatch => ({
  onClickLogout: () => dispatch({ type: LOGOUT }),
  onClickDeleteUser: user =>
    dispatch({ type: DELETE_USER, payload: agent.Auth.delete(user) }),
  onSubmitForm: user =>
    dispatch({ type: SETTINGS_SAVED, payload: agent.Auth.save(user) }),
  onUnload: () => dispatch({ type: SETTINGS_PAGE_UNLOADED })
});


class Settings extends React.Component {

  deleteUser() {
    swal({
      title: "Are you sure?",
      text: "Once deleted, you will not be able to recover your profile! \n All your articles and comments will be deleted!",
      icon: "warning",
      buttons: true,
      dangerMode: true,
    })
    .then((willDelete) => {
      if (willDelete) {
        swal("Poof! Your profile has been deleted!", {
          icon: "success",
          content: this.props.onClickDeleteUser()
        });
      } else {
        swal("Canceled!");
      }
    });
  }

  render() {
    return (
      <div className="settings-page">
        <div className="container page">
          <div className="row">
            <div className="col-md-6 offset-md-3 col-xs-12">

              <h1 className="text-xs-center">Your Settings</h1>

              <SettingsForm
                currentUser={this.props.currentUser}
                onSubmitForm={this.props.onSubmitForm} />

              <ListErrors errors={this.props.errors}></ListErrors>

              <hr />


              <button
                className="btn btn-outline-danger"
                onClick={this.props.onClickLogout}>
                Logout
              </button>

              <button
                className="btn btn-outline-danger pull-xs-right"
                onClick={()=>this.deleteUser()}>
                Delete account
              </button>

            </div>
          </div>
        </div>
      </div>
    );
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(Settings);
